/*
 * Copyright (c) 2000 Dave Jones <dave@powertweak.com>
 *
 * Can be freely distributed and used under the terms of the GNU GPL.
 */

#include <gtk/gtk.h>
#include <procxml.h>
#include "pciutils.h"
#include "ctree.h"
#include "powertweak.h"

extern void Create_XML_SubTree(GtkCTree * tree, GtkCTreeNode * parent, struct menuitem *menuitem);
extern void Create_PCI_SubTree(GtkCTree* tree, GtkCTreeNode* parent);
extern void Create_CPU_SubTree(GtkCTree* tree, GtkCTreeNode* parent);
extern void select_CPU_item(void);
extern void select_PCI_item(struct pci_dev *current_pci_dev);

void free_dataptr (gpointer data)
{
	free(data);
}


void selected_row_callback (GtkCTree *ctree, GtkCTreeNode *node, gpointer __unused foo)
{
//FIXME:
// Yeuch!  The same structure can be used in three different ways.
// And we can only find out the type of structure, by reading it.
// This *may* cause problems.
 
	struct treenode *event;
	struct PCI_treenode *event_PCI;
	struct PROC_treenode *event_PROC;

	event = gtk_ctree_node_get_row_data (GTK_CTREE(ctree), node);
	if (event != NULL) {

		switch (event->node_type) {

			case node_CPU:
				select_CPU_item();
				break;

			case node_PCI:
				event_PCI = gtk_ctree_node_get_row_data (GTK_CTREE(ctree), node);
				select_PCI_item(event_PCI->node_data);
				break;

			case node_PROC_XML:
				event_PROC = gtk_ctree_node_get_row_data (GTK_CTREE(ctree), node);
				select_XML_item(event_PROC->node_xmlmenu);
				break;

			default:
				break;
		}
	}
}


/* Create Hardware subtree */
void Create_Hardware_SubTree(GtkCTree* tree, GtkCTreeNode* parent)
{
	GtkCTreeNode* item_new;
	gchar titlearray[4];
	gchar *title[1];

	title[0] = titlearray;
 
// The following part only works on x86 systems for now, so we ifdef it to
// stop the compiler errors. Messy, but it works.
// One day we'll have an arch/ directory, just like the kernel.
// Until some kinda soul sends me a Sparc or an Alpha, this won't be done (not by me at least) [dj]
// *(Planned for v0.1.15)*

#ifdef __i386__
	/* CPU tree. */
	sprintf (*title, "CPU");
	item_new = gtk_ctree_insert_node (GTK_CTREE(tree), parent, NULL, title, 0, NULL, NULL, NULL, NULL, FALSE, FALSE);
	gtk_ctree_node_set_selectable (GTK_CTREE (tree), item_new, FALSE);
	Create_CPU_SubTree(GTK_CTREE(tree), item_new);
#endif

	/* PCI tree. */
	sprintf (*title, "PCI");
	item_new = gtk_ctree_insert_node (GTK_CTREE(tree), parent, NULL, title, 0, NULL, NULL, NULL, NULL, FALSE, FALSE);
	gtk_ctree_node_set_selectable (GTK_CTREE (tree), item_new, FALSE);
	Create_PCI_SubTree(GTK_CTREE(tree), item_new);
}




/*
 *
 * This is the routine which creates the top level of the tree.
 * It calls all the functions that create all the 'branches' on the tree.
 *
 */

void CreateTree(GtkWidget *scrolled_win)
{
	GtkWidget *tree;
	GtkCTreeNode *item_new;
	gchar titlearray[20];
	gchar *title[1];
	
	struct menuitem *menuitem;

	title[0] = titlearray;
    
	/* Create root tree widget */
	tree = gtk_ctree_new(1, 0);

	gtk_clist_set_row_height(GTK_CLIST (tree), 15);
	gtk_ctree_set_line_style (GTK_CTREE (tree), GTK_CTREE_LINES_DOTTED);
	gtk_ctree_set_expander_style (GTK_CTREE (tree), GTK_CTREE_EXPANDER_SQUARE);
	gtk_clist_set_column_width (GTK_CLIST (tree), 0, 150);
	gtk_ctree_set_indent (GTK_CTREE (tree), 15);
	gtk_clist_set_column_auto_resize (GTK_CLIST (tree), 0, TRUE);
	gtk_clist_set_selection_mode (GTK_CLIST (tree), GTK_SELECTION_BROWSE);
	gtk_signal_connect (GTK_OBJECT (tree), "tree_select_row",
			GTK_SIGNAL_FUNC (selected_row_callback), NULL);


	sprintf (*title, "Hardware");
	item_new = gtk_ctree_insert_node (GTK_CTREE(tree), NULL, NULL, title, 0, NULL, NULL, NULL, NULL, FALSE, FALSE);
	gtk_ctree_node_set_selectable (GTK_CTREE (tree), item_new, FALSE);
	Create_Hardware_SubTree(GTK_CTREE(tree), item_new);

	if (xmlroot==NULL)
		init_xml_subsystem_from_file(XMLFILE);

	menuitem = xmlroot;
	
	while (menuitem!=NULL) {
		/* See if there is anything to add */
		if (menuitem->ActiveCount==0) {
			/* No, there isn't anything */
			menuitem = menuitem->Next;
			continue;
		}
		sprintf (*title, menuitem->Name);
		item_new = gtk_ctree_insert_node (GTK_CTREE(tree), NULL, NULL, title, 0, NULL, NULL, NULL, NULL, FALSE, FALSE);
		gtk_ctree_node_set_selectable (GTK_CTREE (tree), item_new, FALSE);
		Create_XML_SubTree(GTK_CTREE(tree), item_new,menuitem);
		
		menuitem = menuitem->Next;
	}

	
	gtk_container_add (GTK_CONTAINER (scrolled_win), tree);
	gtk_widget_show(tree);
}
