/*
 * Copyright (c) 2000 Dave Jones <dave@powertweak.com>
 *
 * Can be freely distributed and used under the terms of the GNU GPL.
 */

#include <stdio.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include "pciutils.h"
#include "Matrox/known-cards.h"

#define PCI_VENDOR_ID_AMD		0x1022
#define PCI_VENDOR_ID_DEC		0x1011
#define PCI_VENDOR_ID_MATROX		0x102b
#define PCI_VENDOR_ID_UMC		0x1060
#define PCI_VENDOR_ID_VIA		0x1106
#define PCI_VENDOR_ID_Intel		0x8086

#define PCI_DEVICE_ID_AMD_751			0x7006
#define PCI_DEVICE_ID_DEC_21050			0x0001
#define PCI_DEVICE_ID_Intel_82437VX		0x7030
#define PCI_DEVICE_ID_Intel_82439TX		0x7100
#define PCI_DEVICE_ID_Intel_82437FX		0x122d
#define PCI_DEVICE_ID_Intel_82439HX		0x1250
#define PCI_DEVICE_ID_Intel_82443BX		0x7190
#define PCI_DEVICE_ID_Intel_82443BX_NOAGP	0x7192
#define PCI_DEVICE_ID_Intel_82443LX		0x7180
#define PCI_DEVICE_ID_Intel_82424		0x0483
#define PCI_DEVICE_ID_Intel_82430		0x0486
#define PCI_DEVICE_ID_Intel_82434		0x04A3
#define PCI_DEVICE_ID_UMC_8891A			0x0891
#define PCI_DEVICE_ID_VIA_586_0			0x0586
#define PCI_DEVICE_ID_VIA_586_1			0x0571
#define PCI_DEVICE_ID_VIA_597_0			0x0597
#define PCI_DEVICE_ID_VIA_597_1			0x8597
#define PCI_DEVICE_ID_VIA_598_1			0x8598



typedef int (*extended_info_ptr)(GtkWidget *notebook, struct pci_dev *current_pci_dev);
typedef unsigned int (*count_gadgets_ptr)(struct pci_dev *);
typedef void (*init_function_ptr)(struct pci_dev *);
typedef void (*add_gadgets_ptr)(GtkWidget *table, struct pci_dev *, unsigned int numbuttons);
typedef void (*save_function_ptr)(FILE *savehandle, struct pci_dev *);

/*
 * This struct is used to hold information about each device we can get more information
 * out of, or tweak. It's pretty straightforward.
 * If the info/tweak is only valid on one card, we fill in vendor & device, and leave
 * known_cards pointer empty.
 * If we have tweaks/info that applies to many cards (like the Matrox info), then we
 * can do wildcards (sort of), by filling in the vendor, and having known_cards point
 * to a list of device IDs by that vendor which apply. In this case, the device field
 * becomes the number of IDs that are in the device list.
 */
typedef struct devinfo_s {
	unsigned short vendor, device;
	Known_Card_t *known_cards;
	extended_info_ptr extended_info;
	count_gadgets_ptr gadget_counter;
	init_function_ptr init_func;
	add_gadgets_ptr gadget_adder;
	save_function_ptr save_func;
} devinfo_t;
