/*
 * Copyright (c) 2000 Dave Jones <dave@powertweak.com>
 *
 * Can be freely distributed and used under the terms of the GNU GPL.
 */

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <gtk/gtk.h>
#include "pciutils.h"
#include "powertweak.h"
#include "hardware/PCI/known-PCI.h"
#include "hardware/PCI/tweak.h"
#include <procxml.h>


extern struct pci_access *pacc;
extern devinfo_t known_devlist;


extern void show_popup(char *title, char *message);

int writeconfigfile(void)
{
	FILE *configfilehandle;
	struct devinfo_s *thisdev;	
	const char configfilename[]="/etc/powertweak.config";
	const char Comment_header[]="# Powertweak configuration file.\n\
# Automatically generated by Powertweak-config\n\
# Do NOT edit, unless you know what you are doing!\n\n";
	const char PCI_header[]="#[PCI]\n";
	const char PROC_header[]="\n#[PROC]\n";
	struct pci_dev *dev;

	configfilehandle=(fopen(configfilename, "w"));
	if (configfilehandle==NULL)
		return (FALSE);

	/* Save the header. */
	fprintf(configfilehandle, "%s%s", Comment_header, PCI_header);

	/* Now save the settings */
	thisdev = &known_devlist;

	while (thisdev->vendor != 0) {
		dev=pacc->devices;
		while (dev) {
			if ((thisdev->vendor == dev->vendor_id) &&
				 (thisdev->device == dev->device_id) &&
				 (thisdev->save_func != 0)) {
				thisdev->save_func(configfilehandle, dev);
			}
			dev=dev->next;
		}
		thisdev++;
	}

	fprintf(configfilehandle, "%s", PROC_header);
	save_XML(configfilehandle,xmlroot);

	/* Close file, and clean up. */
	fclose(configfilehandle);
	return (TRUE);
}


void savefunction (GtkWidget __unused *widget, gpointer __unused gdata)
{
	char error_message[]="Error trying to save to /etc/powertweak.config !";
	char saved_message[]="Configuration file saved to /etc/powertweak.config";

	if (writeconfigfile()==FALSE)
		show_popup ("Error!", error_message);
	else
		show_popup ("Done.", saved_message);
}


void write_tweaks(FILE *savehandle,
				unsigned int numtweaks,
				Tweaks_t *tweaks,
				struct pci_dev *current_dev)
{
	char name_buffer[256];
	unsigned int i;

	pci_lookup_name(pacc, name_buffer, sizeof(name_buffer),
					PCI_LOOKUP_VENDOR|PCI_LOOKUP_DEVICE,
					current_dev->vendor_id, current_dev->device_id, 0, 0);
	
	fprintf(savehandle, "\n#%s\n", name_buffer);
	for (i=0;i<numtweaks;i++) {
		if (tweaks->current_value == TRUE)
			fprintf (savehandle, "%.4x%.4x_%s\n", current_dev->vendor_id, current_dev->device_id, tweaks->tagname);
		else
			fprintf (savehandle, "#%.4x%.4x_%s\n", current_dev->vendor_id, current_dev->device_id, tweaks->tagname);
		tweaks++;
	}
}


/*
 * The following routine sets everything up to use the current
 * values, by reading status from hardware/configfiles where appropriate
 */
void init_configfile(void)
{
	struct devinfo_s *thisdev;	
	struct pci_dev *dev;

	/* Initialise hardware/PCI entries. */
	thisdev = &known_devlist;
	while (thisdev->vendor != 0) {
		dev=pacc->devices;
		while (dev) {
			if ((thisdev->vendor == dev->vendor_id) &&
				 (thisdev->device == dev->device_id) &&
				 (thisdev->save_func != 0)) {
				thisdev->init_func(dev);
			}
			dev=dev->next;
		}
		thisdev++;
	}

	/* Now for the /proc stuff. */
	if (xmlroot==NULL) 
		init_xml_subsystem_from_file(XMLFILE);
	init_XML_values(xmlroot);
}


void init_tweaks(unsigned int numtweaks, Tweaks_t *tweaks, struct pci_dev *current_dev)
{
	unsigned int i;

	for (i=0;i<numtweaks;i++) {
		if ((pci_read_byte (current_dev, tweaks->reg) & tweaks->mask) == tweaks->value)
			tweaks->original_value=TRUE;
		else
			tweaks->original_value=FALSE;
		tweaks->current_value = tweaks->original_value;
		tweaks++;
	}
}
