/*
 * Hardware tuning for the VIA 82C586_0 PCI-to-ISA bridge
 */

#include <stdio.h>
#include <stdlib.h>
#include "pciutils.h"
#include "../known-PCI.h"
#include "../../dialog.h"
#include "../tweak.h"
#include "82C586-0.h"

#define OPTION_ESCAPE -1
#define OPTION_CHOSEN 0


unsigned int getnumtweaks_VIA_586_0(struct pci_dev *current_dev)
{
	word revision_id;
	revision_id = pci_read_word (current_dev, PCI_REVISION_ID);

	switch (revision_id)  {

		case 0x00 ... 0x3e:
		case 0x3f:
		case 0x40 ... 0x4f:
			return (sizeof(VIA_82C586_0_Tweaks)/sizeof(Tweaks_t))-1;

		// FIXME:This curently detects but does nothing extra to 82C586B (3041 and 3040 silicon)
		default:
			return(0);
	}
}

void init_VIA_586_0(struct pci_dev *current_dev)
{
	unsigned int numtweaks;

	numtweaks = getnumtweaks_VIA_586_0(current_dev);
	if (numtweaks!=0)
		init_tweaks(numtweaks, VIA_82C586_0_Tweaks, current_dev);
}


void tune_VIA_586_0(struct pci_dev *current_dev, unsigned int numbuttons)
{
	word revision_id;
	const char title[]="Powertweak";
	const char prompt[]="Select PCI device to tune.";
	const char *titleptr=title;
	const char *promptptr=prompt;
	const char *currentptr;
	char offstring[]="off";
	int height=20;
	int width=80;
	int retval=666;
	unsigned int chosen_option;
	unsigned int i;

	typedef struct tick_element_s {
		char *tag;
		char *item;
		char *state;
	} tick_element_t;

	Tweaks_t *Tweaks;
	tick_element_t *tick_array;

	/* Subsystem id is programmable on VIA MVP3 so could be anything
	   so it looks as if we have to use the revision id */
	revision_id = pci_read_word (current_dev, PCI_REVISION_ID);

	switch (revision_id)  {

		case 0x00 ... 0x3e:
		case 0x3f:
		case 0x40 ... 0x4f:


			/* Create structure of options. */
			tick_array = malloc((sizeof(tick_element_t)) * numbuttons);
			Tweaks = VIA_82C586_0_Tweaks;

			for (i=0;i<numbuttons;i++) {
				tick_array[i].tag = malloc(16);
				tick_array[i].item = malloc(64);
//FIXME: Here we should read current status.
				tick_array[i].state = offstring;

				sprintf (tick_array[i].tag, "%s", Tweaks->name);
				sprintf (tick_array[i].item, "%s", Tweaks->name);
				Tweaks++;
			}
			init_dialog();

			while (retval != OPTION_ESCAPE) { 
				currentptr = tick_array[0].item;

				/* Show GUI */
				retval = dialog_checklist (titleptr, promptptr, height, width, height-10,
							numbuttons, (const char**)tick_array, FLAG_CHECK, &chosen_option);

				if (retval == OPTION_CHOSEN) {
					printf ("Chosen=%d\n", chosen_option);
				}
			}
			end_dialog();

			/* Destroy structures. */
			printf ("Freeing %d nodes\n", numbuttons);
			for (i=0;i<numbuttons;i++) {
				free (tick_array[i].tag);
				free (tick_array[i].item);
			}
			printf("Freeing array\n");
			free(tick_array);

			break;


		// FIXME:This currently detects but does nothing extra to 82C586B (3041 and 3040 silicon)
		default:
			break;
	}
}


void save_VIA_586_0(FILE *savehandle, struct pci_access *pacc, struct pci_dev *current_dev)
{
	unsigned int numtweaks;

	numtweaks = getnumtweaks_VIA_586_0(current_dev);
	if (numtweaks!=0)
		write_tweaks(savehandle, numtweaks, VIA_82C586_0_Tweaks, current_dev, pacc);
}
