/*
 * Copyright (c) 2000 Dave Jones <dave@powertweak.com>
 *
 * Can be freely distributed and used under the terms of the GNU GPL.
 *
 * DISCLAIMER:
 *	The authors cannot be held responsible for any loss incurred
 *	either directly, or indirectly as a result of running this
 *	utility. Like any other program, there may be bugs lurking
 *	someplace that haven't yet been noticed. As this program
 *	is tweaking low-level chipset registers, anything could happen.
 */

#include <stdio.h>
#include <stdlib.h>
#include "tweak.h"
#include "pciutils.h"
#include "powertweak.h"
#include "../util.h"

/* #include "select-PCI.c" */

#define OPTION_ESCAPE -1
#define OPTION_CHOSEN 0

void select_PCI_item(struct pci_access *pacc, struct pci_dev *current_pci_dev);


unsigned int get_num_PCI_devices(struct pci_access *pacc)
{
	struct pci_dev *dev;
	unsigned int num=0;
	dev = pacc->devices;

	while (dev) {
		num++;
		dev=dev->next;
	}
	return num;
}

int show_PCI_devices(void)
{
	const char title[]="Powertweak";
	const char prompt[]="Select PCI device to tune.";
	const char *titleptr=title;
	const char *promptptr=prompt;
	const char *currentptr;
	char namebuf[128];
	int height=20;
	int width=80;
	int retval=666;	/* rogue value */
	unsigned int chosen_option;
	unsigned int numdevices, i;

	typedef struct menu_element_s {
		char *private;
		char *name;
	} menu_element_t;

	menu_element_t *device_array;

	/* Communication with libpci */
	struct pci_access *pacc;
	struct pci_dev *dev;

	/* Set up the PCI access routines. */
	pacc = pci_alloc();
	pacc->error = die;
	pci_init(pacc);
	pci_scan_bus (pacc);
	
	numdevices=get_num_PCI_devices(pacc);

	/* Create structure of options. */
	device_array = malloc((sizeof(menu_element_t)) * numdevices);
	dev = pacc->devices;

	for (i=0;i<numdevices;i++) {
		device_array[i].private = malloc(10);
		device_array[i].name = malloc(128);

		sprintf (device_array[i].private, "%d", i);
		sprintf (device_array[i].name, "%s",
				pci_lookup_name(pacc, namebuf, sizeof(namebuf),
				PCI_LOOKUP_VENDOR|PCI_LOOKUP_DEVICE,
				dev->vendor_id, dev->device_id, 0, 0 ));
		dev=dev->next;
	}

//	retval = 666; /* rogue value. */
	while (retval != OPTION_ESCAPE) {
		currentptr = device_array[0].name;

		/* Show GUI */
		init_dialog();
		retval = dialog_menu (titleptr, promptptr, height, width, height-10, currentptr, numdevices, FALSE, 
						(const char**)device_array, &chosen_option);
		end_dialog();

		if (retval == OPTION_CHOSEN) {
			dev = pacc->devices;

			for (i=0;i<chosen_option;i++) {
				dev=dev->next;
			}
			select_PCI_item(pacc, dev);
		}
	}

	/* Destroy structures. */
	for (i=0;i<numdevices;i++) {
		free (device_array[i].private);
		free (device_array[i].name);
	}
	free(device_array);

	pci_cleanup(pacc);
	return (0);
}
