/*
 * Copyright (c) 2000 Dave Jones <dave@powertweak.com>
 *
 * Can be freely distributed and used under the terms of the GNU GPL.
 */

/* VENDOR IDs */
#define PCI_VENDOR_ID_DEC		0x1011
#define PCI_VENDOR_ID_MATROX	0x102b
#define PCI_VENDOR_ID_UMC		0x1060
#define PCI_VENDOR_ID_VIA		0x1106
#define PCI_VENDOR_ID_Intel		0x8086

/* PCI DEVICE IDs */
#define PCI_DEVICE_ID_VIA_586_0	0x0586


extern unsigned int getnumtweaks_VIA_586_0 (struct pci_dev *);
extern void init_VIA_586_0 (struct pci_dev *);
extern void tune_VIA_586_0 (struct pci_dev *, unsigned int numbuttons);
extern void save_VIA_586_0 (FILE *, struct pci_access *, struct pci_dev *);


typedef struct Known_Card_s {
	unsigned short device;
} Known_Card_t;


typedef int (*extended_info_ptr)(struct pci_dev *current_pci_dev);
typedef unsigned int (*count_gadgets_ptr)(struct pci_dev *);
typedef void (*init_function_ptr)(struct pci_dev *);
typedef void (*add_gadgets_ptr)(struct pci_dev *, unsigned int numbuttons);
typedef void (*save_function_ptr)(FILE *, struct pci_access *, struct pci_dev *);


/*
 * This struct is used to hold information about each device we can get more information
 * out of, or tweak. It's pretty straightforward.
 * If the info/tweak is only valid on one card, we fill in vendor & device, and leave
 * known_cards pointer empty.
 * If we have tweaks/info that applies to many cards (like the Matrox info), then we
 * can do wildcards (sort of), by filling in the vendor, and having known_cards point
 * to a list of device IDs by that vendor which apply. In this case, the device field
 * becomes the number of IDs that are in the device list.
 */
typedef struct devinfo_s {
	unsigned short vendor, device;
	Known_Card_t *known_cards;
	extended_info_ptr extended_info;
	count_gadgets_ptr gadget_counter;
	init_function_ptr init_func;
	add_gadgets_ptr gadget_adder;
	save_function_ptr save_func;
} devinfo_t;
