/*
 * Copyright (c) 2000 Dave Jones <dave@powertweak.com>
 *
 * Can be freely distributed and used under the terms of the GNU GPL.
 */

#include <stdio.h>
#include <stdlib.h>
#include "pciutils.h"
#include "known-PCI.h"
#include "tweak.h"
#include "powertweak.h"

extern devinfo_t known_devlist;

int writeconfigfile(struct pci_access *pacc)
{
	FILE *configfilehandle;
	struct devinfo_s *thisdev;	
	const char configfilename[]="/etc/powertweak.config";
	const char Comment_header[]="# Powertweak configuration file.\n\
# Automatically generated by Powertweak-config\n\
# Do NOT edit, unless you know what you are doing!\n\n";
	const char PCI_header[]="#[PCI]\n";
	struct pci_dev *dev;
	
	configfilehandle=(fopen(configfilename, "w"));
	if (configfilehandle==NULL)
		return (FALSE);

	/* Save the header. */
	fprintf(configfilehandle, "%s%s", Comment_header, PCI_header);

	/* Now save the settings */
	thisdev = &known_devlist;

	while (thisdev->vendor != 0) {
		dev=pacc->devices;
		while (dev) {
			if ((thisdev->vendor == dev->vendor_id) &&
				 (thisdev->device == dev->device_id) &&
				 (thisdev->save_func != 0)) {
				thisdev->save_func(configfilehandle, pacc, dev);
			}
			dev=dev->next;
		}
		thisdev++;
	}

	/* Close file, and clean up. */
	fclose(configfilehandle);
	return (TRUE);
}


void savefunction (struct pci_access *pacc)
{

//	if (writeconfigfile(pacc)==FALSE)
//		show_popup ("Error!", error_message);
//	else
//		show_popup ("Done.", saved_message);
}


void write_tweaks(FILE *savehandle,
				unsigned int numtweaks,
				Tweaks_t *tweaks,
				struct pci_dev *current_dev,
				struct pci_access *pacc)
{
	char name_buffer[256];
	unsigned int i;

	pci_lookup_name(pacc, name_buffer, sizeof(name_buffer),
					PCI_LOOKUP_VENDOR|PCI_LOOKUP_DEVICE,
					current_dev->vendor_id, current_dev->device_id, 0, 0);
	
	fprintf (savehandle, "\n#%s\n", name_buffer);
	for (i=0;i<numtweaks;i++) {
		if (tweaks->current_value == TRUE)
			fprintf (savehandle, "%.4x%.4x_%s\n", current_dev->vendor_id, current_dev->device_id, tweaks->tagname);
		else
			fprintf (savehandle, "#%.4x%.4x_%s\n", current_dev->vendor_id, current_dev->device_id, tweaks->tagname);
		tweaks++;
	}
}


/*
 * The following routine sets everything up to use the current
 * values, by reading status from hardware/configfiles where appropriate
 */
void init_configfile(struct pci_access *pacc)
{
	struct devinfo_s *thisdev;
	struct pci_dev *dev;

	/* Initialise hardware/PCI entries. */
	thisdev = &known_devlist;
	while (thisdev->vendor != 0) {
		dev=pacc->devices;
		while (dev) {
			if ((thisdev->vendor == dev->vendor_id) &&
				 (thisdev->device == dev->device_id) &&
				 (thisdev->save_func != 0)) {
				thisdev->init_func(dev);
			}
			dev=dev->next;
		}
		thisdev++;
	}
}
