/*

	Datastructure-manipulation for the /proc items, XML based

	(C) 2000 Arjan van de Ven, licensed under the terms of the GPL
                                                                         
*/

#include <stdlib.h>
#include <stdio.h>
#include <parser.h>		/* libxml */
#include <string.h>
#include <assert.h>
#include <gtk/gtk.h>

#include <procxml.h>
#include <powertweak.h>

struct menuitem *xmlroot = NULL;

struct menuitem *findinlist(char *Item, struct menuitem *menu)
{
	if (Item == NULL)
		return NULL;
	if (menu == NULL)
		return NULL;

	while (menu != NULL) {
		if ((menu->Name!=NULL)&&(strcasecmp(Item, menu->Name) == 0))
			return menu;
		menu = menu->Next;
	}
	return NULL;
}

static struct menuitem *additemtolist(char *Item, struct menuitem **menu)
{
	struct menuitem *MenuItem;
	struct menuitem **Prev, *current;

	Prev = menu;
	current = *menu;
	
	if (Item==NULL)
		return NULL;


	while (current != NULL) {
		if ((current->Name!=NULL)&&(strcasecmp(Item, current->Name) == 0))
			return current;
		Prev = &(current->Next);
		current = current->Next;
	}

	MenuItem = malloc(sizeof(struct menuitem));
	if (MenuItem==NULL) {
		printf("Out of memory\n");
		exit(0);
	}
		
	memset(MenuItem, 0, sizeof(struct menuitem));
	MenuItem->Name = strdup(Item);
	MenuItem->Next = NULL;
	*Prev = MenuItem;

	return MenuItem;
}

void __add_xml_item(char *Menu, char *SubMenu, char *Tab, struct xml_item *xml_item, struct menuitem **rootmenu)
{
	struct menuitem *temp;
	
	if ((xml_item==NULL)||(xml_item->Name==NULL))
		return;

	if (Menu == NULL) {
		temp = additemtolist(xml_item->Name, rootmenu);
		temp->xml_item = xml_item;
	} else {
		temp = additemtolist(Menu, rootmenu);
		__add_xml_item(SubMenu, Tab, NULL, xml_item, &(temp->Submenus));
	}
}

void add_xml_item(struct xml_item *item, struct menuitem **rootmenu)
{
	assert(item != NULL);
	__add_xml_item(item->Menu, item->SubMenu, item->Tab, item, rootmenu);
}

struct xml_item *parse_item(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
{
	struct xml_item *xml_item = NULL;
	
	if (ns!=NULL) {}; /* Prevent a warning. "ns" is for future use */

	xml_item = malloc(sizeof(struct xml_item));
	if (xml_item == NULL) {
		printf("Out of memory\n");
		return NULL;
	}
	memset(xml_item, 0, sizeof(struct xml_item));

	cur = cur->childs;
	while (cur != NULL) {
		if (!strcasecmp(cur->name, "MENU"))
			xml_item->Menu = xmlNodeListGetString(doc, cur->childs, 1);
		if (!strcasecmp(cur->name, "SUBMENU"))
			xml_item->SubMenu = xmlNodeListGetString(doc, cur->childs, 1);
		if (!strcasecmp(cur->name, "TAB"))
			xml_item->Tab = xmlNodeListGetString(doc, cur->childs, 1);
		if (!strcasecmp(cur->name, "NAME"))
			xml_item->Name = xmlNodeListGetString(doc, cur->childs, 1);
		if (!strcasecmp(cur->name, "CONFIGNAME"))
			xml_item->ConfigName = xmlNodeListGetString(doc, cur->childs, 1);
		if (!strcasecmp(cur->name, "TYPE"))
			xml_item->Type = xmlNodeListGetString(doc, cur->childs, 1);
		if (!strcasecmp(cur->name, "FILE"))
			xml_item->File = xmlNodeListGetString(doc, cur->childs, 1);
		if (!strcasecmp(cur->name, "ELEMENT"))
			xml_item->Element = atoi(xmlNodeListGetString(doc, cur->childs, 1));
		if (!strcasecmp(cur->name, "LOW"))
			xml_item->MinValue = atoi(xmlNodeListGetString(doc, cur->childs, 1));
		if (!strcasecmp(cur->name, "HIGH"))
			xml_item->MaxValue = atoi(xmlNodeListGetString(doc, cur->childs, 1));
		if (!strcasecmp(cur->name, "DESCRIPTION"))
			xml_item->Description = xmlNodeListGetString(doc, cur->childs, 1);
		cur = cur->next;

	}

	return xml_item;
}


struct menuitem *parse_file(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
{
	struct menuitem *Root = NULL;
	
	if (cur==NULL)
		return NULL;
		
	cur = cur->childs;
	while (cur != NULL) {
		
		if ((cur->name!=NULL)&&(strcasecmp(cur->name, "PROCENTRY")==0))
			add_xml_item(parse_item(doc, ns, cur), &Root);
		cur = cur->next;
	}

	return Root;
}


/* printRecurse is for debugging only */
void printRecurse(struct menuitem *menu, int depth)
{
	int i;

	while (menu != NULL) {
		for (i = 0; i < depth; i++)
			printf("  ");
		printf("%s", menu->Name);

		if (menu->xml_item != NULL)
			printf(" (%i,%i) ", menu->xml_item->MinValue, menu->xml_item->MaxValue);
			
		printf(" [%i] ",menu->ActiveCount);
		printf("\n");

		if (menu->Submenus != NULL)
			printRecurse(menu->Submenus, depth + 1);

		menu = menu->Next;
	}


}

/* 
 *	calculate_node_usage counts the number of "active" /proc items, ie. items
 *	that actually exist. It handles recursion in the obvious way:
 *      An node is assigned a count of "1" (if the /proc file exists) plus
 *      the sum of the nodes children.
 *
 * 	The return-value is the nodes value plus the value of all the ->Next neighbors
 *      so that you can say "  calculate_node_usage(menuitem->Submenus)" for the
 *      return-value to make sense.  
 *
 */
 
int calculate_node_usage(struct menuitem *menuitem)
{
	int temp;
	
	temp = 0;
	
	if (menuitem==NULL)
		return 0;
		
	if ((menuitem->xml_item!=NULL)&&(menuitem->xml_item->File!=NULL)) {
		if (fileexists(menuitem->xml_item->File)==TRUE)
			temp++;
	}
	
	/* Count all child-nodes */
	temp += calculate_node_usage(menuitem->Submenus);
	
	/* Update the local field */
	menuitem->ActiveCount = temp;

	/* Traverse the list */
	temp +=	 calculate_node_usage(menuitem->Next);
	
	return temp;
	
}


void init_xml_subsystem_from_file(char *Filename)
{
	xmlDocPtr doc;
	xmlNodePtr cur;
	
	if (Filename==NULL)
		return;

	doc = xmlParseFile(Filename);
	if (doc == NULL) {
		printf("Severe XML error: doc == NULL!!");
		return;
	}

	cur = doc->root;
	if (cur == NULL) {
		printf("Severe XML error: cur == NULL");
		return;
	}

	xmlroot = parse_file(doc, NULL, cur);

	/* Calculate the actual number of /proc items present.
	   (void) because we don't care for the total, only the subtotals. */
	   
	(void)calculate_node_usage(xmlroot);
	
#ifdef DEBUG
	/* Debug code: print the tree to the screen */
	printRecurse(xmlroot, 0);
#endif


}

void init_XML_values(struct menuitem *menuitem)
{
	while (menuitem != NULL) {
		if (menuitem->xml_item != NULL)
			if (fileexists(menuitem->xml_item->File) == TRUE) {
				if (strcmp(menuitem->xml_item->Type, "Spinbutton") == 0)
					menuitem->xml_item->IntValue =
					    read_int_from_proc_file(menuitem->xml_item->File,
								    menuitem->xml_item->Element);
				if (strcmp(menuitem->xml_item->Type, "Checkbox") == 0)
					menuitem->xml_item->BoolValue =
					    read_int_from_proc_file(menuitem->xml_item->File,
								    menuitem->xml_item->Element);
			}
		if (menuitem->Submenus != NULL)
			init_XML_values(menuitem->Submenus);
		menuitem = menuitem->Next;
	}
}


void save_XML(FILE * savehandle, struct menuitem *menuitem)
{
//	printf("Saving\n");
	while (menuitem != NULL) {
		if (menuitem->xml_item != NULL) {
			if (strcmp(menuitem->xml_item->Type, "Checkbox") == 0)
				menuitem->xml_item->IntValue = menuitem->xml_item->BoolValue;
			fprintf(savehandle, "PROC_%s=%d\n",
				menuitem->xml_item->ConfigName, menuitem->xml_item->IntValue);

		}
		if (menuitem->Submenus != NULL)
			save_XML(savehandle, menuitem->Submenus);

		menuitem = menuitem->Next;
	}
}
