/*
 * Copyright (c) 2000 Dave Jones <dave@powertweak.com>
 * Can be freely distributed and used under the terms of the GNU GPL.
 * 
 * Routines to convert config file tags into pokes.
 */

#include <stdio.h>
#include <stdlib.h>
#include "pciutils.h"
#include "tweak.h"
#include "powertweak.h"

#define OPTION_UNSET -1
#define OPTION_OFF	0
#define OPTION_ON	1

extern int find_PCI_tag(unsigned long device_id, const char *tagname, unsigned char *result);
extern int open_log_file(FILE **fp);
extern void close_log_file(FILE **fp);

int tune_device(Tweaks_t *Tweaks, struct pci_dev *current_dev)
{
	FILE *logfile;
	int tweaksmade=0;
	unsigned char current_value, option, actual_poke_value;
	unsigned long hardware_ID;

	hardware_ID = (current_dev->vendor_id<<16)|current_dev->device_id;

	while (Tweaks->tagname) {

		switch (find_PCI_tag(hardware_ID, Tweaks->tagname, &option)) {

			/* Tag found. */
			case TRUE:
				current_value = pci_read_byte (current_dev, Tweaks->poke_reg);
				actual_poke_value = ((current_value & (~Tweaks->poke_mask)) | Tweaks->poke_value);

				if (current_value != actual_poke_value) {
					if (open_log_file(&logfile)==TRUE) {
						fprintf(logfile, "\tPOKE:%s enabled.\n", Tweaks->poke_name);
						fprintf(logfile, "Register: 0x%x\tBefore: 0x%x\tAfter: 0x%x\n",
							Tweaks->poke_reg, current_value, actual_poke_value);
						close_log_file(&logfile);
					}
					pci_write_byte (current_dev, Tweaks->poke_reg, actual_poke_value);

					tweaksmade++;
				}
				break;

			/* Tag found, but with no parameter. */
			case OPTION_UNSET:
				printf ("Powertweak hasn't been configured properly\n");
				return(1);

			/* Tag not found, skip to next tweak. */
			case FALSE:
				break;

		}
		Tweaks++;
	}

	if (tweaksmade==0) {
		if (open_log_file(&logfile)==TRUE) {
			fprintf(logfile, "\t-> Device is already tuned.\n");
			close_log_file(&logfile);
		}
	}
	return(0);
}
