/*
 * Copyright (c) 2000 Dave Jones <dave@powertweak.com>
 *
 * Can be freely distributed and used under the terms of the GNU GPL.
 *
 *
 * This routine scans through an array of strings, checking if
 * they exist as /proc/sys entries, and if so, writes the value
 * from the config file to them.
 *
 * Note, that entries may not exist due to old kernel, or support for that
 *  feature not being compiled into the current running kernel)
 */

#include <stdio.h>
#include <stdlib.h>
#include "powertweak.h"

extern int find_PROC_tag(const char *, char **);
extern void tune_vm_bdflush(void);
extern void tune_vm_buffermem(void);
extern void tune_vm_freepages(void);
extern void tune_vm_kswapd(void);
extern void tune_vm_pagecache(void);
extern void tune_vm_pagetable_cache(void);

//FIXME: The strings holding the filename shouldn't need /proc/sys in them.
//		 Instead just copy the non-const part into an array
//		 prefixed by /proc/sys
//FIXME: Implement range checking.

void Optimise_proc(void)
{
	const char *proc_names[] = {
		"CDROM-AUTOCLOSE", "/proc/sys/dev/cdrom/autoclose",
		"CDROM-AUTOEJECT", "/proc/sys/dev/cdrom/autoeject",
		"CDROM-CHECK-MEDIA", "/proc/sys/dev/cdrom/check_media",
		"CDROM-LOCK", "/proc/sys/dev/cdrom/lock",
		"CDROM-DEBUG", "/proc/sys/dev/cdrom/debug",

		"FS-DQUOT-MAX",	"/proc/sys/fs/dquot-max",
		"FS-FILE-MAX",	"/proc/sys/fs/file-max",
		"FS-INODE-MAX",	"/proc/sys/fs/inode-max",
		"FS-SUPER-MAX",	"/proc/sys/fs/super-max",

		"NET-CORE-RMEM-DEFAULT",	"/proc/sys/net/core/rmem_default",
		"NET-CORE-RMEM-MAX",		"/proc/sys/net/core/rmem_max",
		"NET-CORE-WMEM-DEFAULT",	"/proc/sys/net/core/wmem_default",
		"NET-CORE-WMEM-MAX",		"/proc/sys/net/core/wmem_max",

		"NET-ICMP-DESTUNREACHRATE",		"/proc/sys/net/ipv4/icmp_destunreach_rate",
		"NET-ICMP-ECHOREPLYRATE",		"/proc/sys/net/ipv4/icmp_echoreply_rate",
		"NET-ICMP-PARAMPROBRATE",		"/proc/sys/net/ipv4/icmp_paramprob_rate",
		"NET-ICMP-TIMEEXCEEDRATE",		"/proc/sys/net/ipv4/icmp_timeexceed_rate",
		"NET-ICMP-IGNOREBOGUSERRORS",	"/proc/sys/net/ipv4/icmp_ignore_bogus_error_responses",
		"NET-ICMP-ECHOIGNOREALL",		"/proc/sys/net/ipv4/icmp_echo_ignore_all",
		"NET-ICMP-ECHOIGNOREBROADCASTS","/proc/sys/net/ipv4/icmp_echo_ignore_broadcasts",

		"NET-IP-DEFAULT-TTL",	"/proc/sys/net/ipv4/ip_default_ttl",
		"NET-IP-FRAGTIME",		"/proc/sys/net/ipv4/ipfrag_time",
		"NET-IP-NO-PMTU",		"/proc/sys/net/ipv4/ip_no_pmtu_disc",
		"NET-IP-ALWAYS-DEFRAG",	"/proc/sys/net/ipv4/ip_always_defrag",
		"NET-IP-MASQ-DEBUG",	"/proc/sys/net/ipv4/ip_masq_debug",

		"NET-TCP-FIN-TIMEOUT",		"/proc/sys/net/ipv4/tcp_fin_timeout",
		"NET-TCP-KEEPALIVE-TIME",	"/proc/sys/net/ipv4/tcp_keepalive_time",
		"NET-TCP-KEEPALIVE-PROBES",	"/proc/sys/net/ipv4/tcp_keepalive_probes",
		"NET-TCP-MAX-KEEPALIVES",	"/proc/sys/net/ipv4/tcp_max_ka_probes",
		"NET-TCP-SYN-RETRIES",		"/proc/sys/net/ipv4/tcp_syn_retries",
		"NET-TCP-MAX-SYN-BACKLOG",	"/proc/sys/net/ipv4/tcp_max_syn_backlog",
		"NET-TCP-RETRIES1",			"/proc/sys/net/ipv4/tcp_retries1",
		"NET-TCP-RETRIES2",			"/proc/sys/net/ipv4/tcp_retries2",
		"NET-TCP-SYNCOOKIES",		"/proc/sys/net/ipv4/tcp_syncookies",
		"NET-TCP-RETRANS-COLLAPSE",	"/proc/sys/net/ipv4/tcp_retrans_collapse",
		"NET-TCP-SACK",				"/proc/sys/net/ipv4/tcp_sack",
		"NET-TCP-TIMESTAMPS",		"/proc/sys/net/ipv4/tcp_timestamps",
		"NET-TCP-WINDOW-SCALING",	"/proc/sys/net/ipv4/tcp_window_scaling",

		"NET-UNIX-DELETE-DELAY",	"/proc/sys/net/unix/delete_delay",
		"NET-UNIX-DESTROY-DELAY",	"/proc/sys/net/unix/destroy_delay",
		"NET-UNIX-MAX-DGRAM-QLEN",	"/proc/sys/net/unix/max_dgram_qlen",

		"NET-TOKENRING-RIF-TIMEOUT","/proc/sys/net/token-ring/rif_timeout",

		"KERNEL-CTRL-ALT-DEL",				"/proc/sys/kernel/ctrl-alt-del",
		"KERNEL-SYSRQ",						"/proc/sys/kernel/sysrq",
		"KERNEL-PANIC",						"/proc/sys/kernel/panic",
		"KERNEL-ACCT-RESUME",				"/proc/sys/kernel/acct",
		"KERNEL-ACCT-SUSPEND",				"/proc/sys/kernel/acct",
		"KERNEL-ACCT-TIMER",				"/proc/sys/kernel/acct",
		"KERNEL-PRINTK-CONSOLE-LOGLEVEL",	"/proc/sys/kernel/printk",
		"KERNEL-RTSIG-MAX",					"/proc/sys/kernel/rtsig-max",
		"KERNEL-SHMALL",					"/proc/sys/kernel/shmall",
		"KERNEL-SHMMAX",					"proc/sys/kernel/shmmax",

		"VM-VM2-PAGE-CLUSTER",		"/proc/sys/vm/page-cluster",
		
	0 }; 

	const char **name_ptr;
	char *option;

	FILE *file=0;

	name_ptr = proc_names;

	while (*name_ptr) {
		/* Is this tweak in the config file ? */
		if (find_PROC_tag(*name_ptr, &option)==TRUE) {

			name_ptr++;
			/* Does the /proc/sys entry exist? */
			file = fopen (*name_ptr, "w");
			if (file!=NULL) {
				fprintf(file, "%s\n", option);
				fclose (file);
			}
			name_ptr++;
		} else {
			name_ptr+=2;
		}
	}

	tune_vm_bdflush();
	tune_vm_buffermem();
	tune_vm_freepages();
	tune_vm_kswapd();
	tune_vm_pagecache();
	tune_vm_pagetable_cache();

}

