/*
 * Copyright (c) 2000 Dave Jones <dave@powertweak.com>
 *
 * Can be freely distributed and used under the terms of the GNU GPL.
 *
 *
 * This routine scans through an array of strings, checking if
 * they exist as /proc/sys entries, and if so, writes the value
 * from the config file to them.
 *
 * Note, that entries may not exist due to old kernel, or support for that
 *  feature not being compiled into the current running kernel)
 */


//FIXME: This should all be using sysctls really.
//FIXME: The strings holding the filename shouldn't need /proc/sys in them.
//		 Instead just copy the non-const part into an array
//		 prefixed by /proc/sys
//FIXME: Implement range checking.

#include <stdio.h>
#include <stdlib.h>
#include "powertweak.h"

extern char *read_char_from_proc_file(const char *, int);
extern unsigned int read_int_from_proc_file(const char *, int);
extern int find_PROC_tag(const char*, char **);

void tune_vm(char *filename, const char *tags[], unsigned int num)
{
	char *position;
	char string[80];
	FILE *file=0;
	unsigned int offset=0;
	char *c_option;
	int i_option;

	position = string;

	for (offset=0; offset<num; offset++) {

		if (tags[offset]==0) {
			i_option = read_int_from_proc_file(filename, offset);
			position += sprintf (position, "%d ", i_option);

		} else {

			/* If we have no entry for this element,
			 * we read the current value from /proc */
			if (find_PROC_tag(tags[offset], &c_option)==FALSE) {
					printf ("Falling back to /proc for %s\n", tags[offset]);
				c_option = read_char_from_proc_file (filename, offset);
			}
			position += sprintf (position, "%s ", c_option);
		}
	}
	sprintf (position, "\n");

	/* Does the /proc/sys entry exist? */
	file = fopen (filename, "w");
	if (file!=NULL) {
		fprintf(file, "%s\n", string);
		fclose (file);
	}
}


void tune_vm_bdflush(void)
{
	const char *bdflush_tags[] = {
		"VM-BDFLUSH-NFRACT",
		"VM-BDFLUSH-NDIRTY",
		"VM-BDFLUSH-NREFILL",
		"VM-BDFLUSH-NREF-DIRT",
		0,
		"VM-BDFLUSH-AGE-BUFFER",
		"VM-BDFLUSH-AGE-SUPER",
		0,
		0 }; 
#define NUM_OF_BDFLUSH_TWEAKS sizeof (bdflush_tags)/sizeof(char *)
	tune_vm("/proc/sys/vm/bdflush", bdflush_tags, NUM_OF_BDFLUSH_TWEAKS);
}


void tune_vm_buffermem(void)
{
	const char *buffermem_tags[] = {
		"VM-BUFFERMEM-MIN",
		0,
		"VM-BUFFERMEM-MAX" }; 
#define NUM_OF_BUFFERMEM_TWEAKS sizeof (buffermem_tags)/sizeof(char *)
	tune_vm("/proc/sys/vm/buffermem", buffermem_tags, NUM_OF_BUFFERMEM_TWEAKS);
}


void tune_vm_freepages(void)
{
	const char *freepages_tags[] = {
		"VM-FREEPAGES-MIN",
		"VM-FREEPAGES-LOW",
		"VM-FREEPAGES-HIGH" }; 
#define NUM_OF_FREEPAGES_TWEAKS sizeof (freepages_tags)/sizeof(char *)
	tune_vm("/proc/sys/vm/freepages", freepages_tags, NUM_OF_FREEPAGES_TWEAKS);
}


void tune_vm_kswapd(void)
{
	const char *kswapd_tags[] = {
		"VM-KSWAPD-TRIES-BASE",
		"VM-KSWAPD-TRIES-MIN",
		"VM-KSWAPD-SWAP-CLUSTER" }; 
#define NUM_OF_KSWAPD_TWEAKS sizeof (kswapd_tags)/sizeof(char *)
	tune_vm("/proc/sys/vm/kswapd", kswapd_tags, NUM_OF_KSWAPD_TWEAKS);
}


void tune_vm_pagecache(void)
{
	const char *pagecache_tags[] = {
		"VM-PAGECACHE-MIN",
		0,
		"VM-PAGECACHE-MAX" }; 
#define NUM_OF_PAGECACHE_TWEAKS sizeof (pagecache_tags)/sizeof(char *)
	tune_vm("/proc/sys/vm/pagecache", pagecache_tags, NUM_OF_PAGECACHE_TWEAKS);
}

void tune_vm_pagetable_cache(void)
{
	const char *pagetablecache_tags[] = {
		"VM-PAGETABLE-CACHE-MIN",
		"VM-PAGETABLE-CACHE-MAX" }; 
#define NUM_OF_PAGETABLECACHE_TWEAKS sizeof (pagetablecache_tags)/sizeof(char *)
	tune_vm("/proc/sys/vm/pagetable_cache", pagetablecache_tags, NUM_OF_PAGETABLECACHE_TWEAKS);
}
